#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 2A.ml - 2016 - Compétition - Importation des données
# 
# Une compétition était proposée dans le cadre du cours *Python pour un Data Scientist* à l'[ENSAE](http://www.ensae.fr/ensae/fr/). Ce notebook facilite la prise en main des données et étudie les données avec des méthodes de statistiques descriptives.

# In[1]:


get_ipython().run_line_magic('matplotlib', 'inline')
from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


from pyensae.datasource import download_data
download_data("ensae_competition_2016.zip",
              url="https://github.com/sdpython/ensae_teaching_cs/raw/master/_doc/competitions/2016_ENSAE_2A/")


# ## Données

# In[3]:


import pandas as p
df = p.read_csv('./ensae_competition_train.txt', header=[0,1], sep="\t", index_col=0)
df.head()


# In[4]:


len(df)


# This research employed a binary variable, default payment (Yes = 1, No = 0), as the response variable. This study reviewed the literature and used the following 23 variables as explanatory variables: 
# - X1: Amount of the given credit (NT dollar): it includes both the individual consumer credit and his/her family (supplementary) credit. 
# - X2: Gender (1 = male; 2 = female). 
# - X3: Education (1 = graduate school; 2 = university; 3 = high school; 4 = others). 
# - X4: Marital status (1 = married; 2 = single; 3 = others). 
# - X5: Age (year). 
# - X6 - X11: History of past payment. We tracked the past monthly payment records (from April to September, 2005) as follows: X6 = the repayment status in September, 2005; X7 = the repayment status in August, 2005; . . .;X11 = the repayment status in April, 2005. The measurement scale for the repayment status is: -1 = pay duly; 1 = payment delay for one month; 2 = payment delay for two months; . . .; 8 = payment delay for eight months; 9 = payment delay for nine months and above. 
# - X12-X17: Amount of bill statement (NT dollar). X12 = amount of bill statement in September, 2005; X13 = amount of bill statement in August, 2005; . . .; X17 = amount of bill statement in April, 2005. 
# - X18-X23: Amount of previous payment (NT dollar). X18 = amount paid in September, 2005; X19 = amount paid in August, 2005; . . .;X23 = amount paid in April, 2005. 

# In[5]:


#Format des variables : multiindex
print(df[('Y','default payment next month')].value_counts())
print(df['Y']['default payment next month'].value_counts())


# In[6]:


df.columns = df.columns.droplevel()
df.rename(columns={'default payment next month':'Y'},inplace=True)


# In[7]:


df.head()


# ## Distinction des variables catégorielles

# In[8]:


df.columns


# In[9]:


cat_var =  list(df.columns[1:4]) + list(df.columns[5:11]) + list(df.columns[-1:])

num_var = [v for v in df.columns if v not in cat_var]

print("Categorical variables: ", cat_var)
print("Numerical variables: "  , num_var)

for v in cat_var[:-1]:
    df[v]=p.DataFrame(df[v]).applymap(str)


# In[10]:


num_var_bill = [ v for v in num_var if 'BILL' in v or v == 'LIMIT_BAL']
num_var_pay  = [ v for v in num_var if 'PAY' in v ]
print("Numerical variables BILL: ", num_var_bill)
print("Numerical variables PAY: " , num_var_pay)


# ## Variable d'intérêt

# In[11]:


default=df[df['Y']==1]
solvent=df[df['Y']==0]
df['Y'].value_counts()


# ## Statistiques Descriptives

# ### Variables quantitatives 

# In[12]:


desc=df[num_var].describe()
std=p.DataFrame(df[num_var].std()).transpose()
std.rename(index={0:'std'},inplace=True)
res=p.concat([desc,std])
res


# In[13]:


from pandas import DataFrame
import matplotlib.pyplot as plt
import seaborn as sns
# from seaborn.timeseries import tsplot as tsplot
from seaborn import lineplot as tsplot
import numpy as np

fig = plt.figure(figsize=(20,10))

ax1 = fig.add_subplot(121)
ax2 = fig.add_subplot(122)

sns.set(style="ticks")

ax1.set_ylim([15000,25000])
ax2.set_ylim([1000,2500])

ax1.set_title('BILL variables',fontsize=14)
ax2.set_title('PAY variables',fontsize=14)



df_bill = DataFrame(data=df[num_var_bill[1:]].values.ravel(), columns=['bill'])
df_bill['index'] = [(i % 6 + 1) for i in range(df_bill.shape[0])]
tsplot(x='index', y='bill',
       estimator=np.median, ax=ax1, data=df_bill)

df_pay = DataFrame(data=df[num_var_pay].values.ravel(), columns=['pay'])
df_pay['index'] = [(i % 6 + 1) for i in range(df_bill.shape[0])]
tsplot(x='index', y='pay',
       estimator=np.median, ax=ax2, data=df_pay);


# In[14]:


desc=default[num_var].describe()
std=p.DataFrame(default[num_var].std()).transpose()
std.rename(index={0:'std'},inplace=True)
p.concat([desc,std])


# In[15]:


desc=solvent[num_var].describe()
std=p.DataFrame(solvent[num_var].std()).transpose()
std.rename(index={0:'std'},inplace=True)
p.concat([desc,std])


# Biais de sélection sur les clients suceptibles de faire défaut ? Ou effet de taille (catégorie moins nombreuse) => moins de chances d'avoir des crédits élevés.

# In[16]:


sample=solvent[num_var].sample(len(default))

desc=sample.describe()
std=p.DataFrame(sample.std()).transpose()
std.rename(index={0:'std'},inplace=True)
p.concat([desc,std])


# Il semble y avoir un biais de sélection en amont.

# In[17]:


import matplotlib.pyplot as plt
import seaborn as sns
import numpy as np

fig, axes = plt.subplots(3,2, sharex=True)
fig = plt.figure(figsize=(20,10))

sns.set(style="ticks")

df_dict={0: 'All - ', 1 : 'Default - ' , 2 : 'Solvent (sample) - '}
var_dict={0: 'BILL variables', 1 : 'PAY variables'}

for i in range(3):
    for j in range(2):
        axes[i,j].set_title(df_dict[i] + var_dict[j],fontsize=8)

        
df_bill = DataFrame(data=df[num_var_bill[1:]].values.ravel(), columns=['bill'])
df_bill['index'] = [(i % 6 + 1) for i in range(df_bill.shape[0])]
tsplot(x='index', y='bill', estimator=np.median, ax=axes[0,0], data=df_bill)

df_pay = DataFrame(data=df[num_var_pay].values.ravel(), columns=['pay'])
df_pay['index'] = [(i % 6 + 1) for i in range(df_bill.shape[0])]
tsplot(x='index', y='pay', estimator=np.median, ax=axes[0,1], data=df_pay)

df_bill = DataFrame(data=default[num_var_bill[1:]].values.ravel(), columns=['bill'])
df_bill['index'] = [(i % 6 + 1) for i in range(df_bill.shape[0])]
tsplot(x='index', y='bill', estimator=np.median, ax=axes[1,0], data=df_bill)

df_pay = DataFrame(data=default[num_var_pay].values.ravel(), columns=['pay'])
df_pay['index'] = [(i % 6 + 1) for i in range(df_bill.shape[0])]
tsplot(x='index', y='pay', estimator=np.median, ax=axes[1,1], data=df_pay)

df_bill = DataFrame(data=sample[num_var_bill[1:]].values.ravel(), columns=['bill'])
df_bill['index'] = [(i % 6 + 1) for i in range(df_bill.shape[0])]
tsplot(x='index', y='bill', estimator=np.median, ax=axes[2,0], data=df_bill)

df_pay = DataFrame(data=sample[num_var_pay].values.ravel(), columns=['pay'])
df_pay['index'] = [(i % 6 + 1) for i in range(df_bill.shape[0])]
tsplot(x='index', y='pay', estimator=np.median, ax=axes[2,1], data=df_pay);


# In[18]:


df_resample = p.concat([df[df['Y']==1], df[df['Y']==0].sample(len(df[df['Y']==1]))])

for v in cat_var[:-1]:
    print("Variable {}\n".format(v), p.crosstab(df_resample['Y'],df_resample[v]))


# ### Variables numériques (sur un sample d'individus)

# In[19]:


get_ipython().run_line_magic('matplotlib', 'inline')


# In[20]:


import matplotlib.pyplot as plt

fig, axes = plt.subplots(5,3, sharex=True, figsize=(12,16))

for i in range(5):
    for j in range(3):
        if i*3+j < len(num_var) :
            axes[i,j].set_title(num_var[i*3+j],fontsize=8)
            sns.violinplot(x="Y", y=num_var[i*3+j], \
                        data=df_resample, palette=["#2F6CE6","#E62F2F"],  ax = axes[i,j])
            #sns.stripplot(x="Y", y=num_var[i*3+j], \
            #            data=df_resample, palette=["#2F6CE6","#E62F2F"], jitter=True,  ax = axes[i,j])
            axes[i,j].set_ylabel('')
            yticks=axes[i,j].get_yticks()
            axes[i,j].set_yticks(yticks[::2])


# In[21]:


import seaborn as sns

sns.set(style="ticks")
sns.pairplot(df_resample[num_var_bill + ['Y']], hue='Y', palette=["#2F6CE6","#E62F2F"]);


# In[22]:


import seaborn as sns

sns.set(style="ticks")
sns.pairplot(df_resample[num_var_pay + ['Y']], hue='Y', palette=["#2F6CE6","#E62F2F"]);


# Idées pouvant être explorées : 
#  - le default doit pouvoir se repérer dans la décélération des remboursements. 
#  - il y a aussi peut etre un effet "biais du survivant" : les users qui sont repérés comme susceptibles d'entraîner un défaut => montant de crédits accordés plus petits. 
#  - multicolinéarité de nombreuses variables => faire une ACP au préalable